import GLib from 'gi://GLib';
import Gio from 'gi://Gio';
import St from 'gi://St';
import Meta from 'gi://Meta';
import Shell from 'gi://Shell';
import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';

export default class BlurtExtension extends Extension {
    
    enable() {     
        
        this._siButton = new St.Button(
        {
            style_class : "panel-button"
        });
        this._siLabel = new St.Label({
            text: '\u{0181}',
            style_class: 'si-label',
        });
        this._siButton.set_child(this._siLabel);
        this._siButton.connect('clicked', async () => this.transcribe());
        Main.panel._rightBox.insert_child_at_index(this._siButton, 1);
        this._settings = this.getSettings();
	this._subproc = null;
	this._stopped = false;
        Main.wm.addKeybinding("speech-input", 
            this._settings,
            Meta.KeyBindingFlags.IGNORE_AUTOREPEAT,
            Shell.ActionMode.NORMAL,
	    async () => this.transcribe()
        );
    }

    async transcribe() {
	if (this._subproc == null) {
	    this._stopped = false;
            try {
                this._siLabel.set_style_class_name('i-label');
                const asr_path = this._settings.get_string('whisper-path');
                this._subproc = Gio.Subprocess.new([asr_path],
                    Gio.SubprocessFlags.NONE);
                const success = await this._subproc.wait_check_async(null);
                this._siLabel.set_style_class_name(success ? 'si-label' : 'e-label');
            } catch (e) {
                logError(e);
                this._siLabel.set_style_class_name(this._stopped ? 'si-label' : 'e-label');
            }
	    this._subproc = null;
	} else {
	    this._stopped = true;
	    this._subproc.send_signal(1);
	}
    }
    
    disable() {
        Main.wm.removeKeybinding("speech-input");
        Main.panel._rightBox.remove_child(this._siButton);
        this._siLabel = null;
        this._siButton = null;
        this._settings = null;
    }
    
}
